"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const RuntimePrivate_1 = require("./RuntimePrivate");
var ServiceDebugging;
(function (ServiceDebugging) {
    /** Enable debug logging by passing true, or a filtering regex. */
    function enableLogging(filter) {
        let mode = "";
        if (filter === false) {
            // Don't show anything in the console if logging gets explicitly disabled at startup
            if (wantsVerboseLogging === false)
                return;
            mode = "default (errors only)";
            wantsVerboseLogging = false;
        }
        else if (filter === true) {
            mode = "verbose";
            wantsVerboseLogging = true;
        }
        else if (filter instanceof RegExp) {
            mode = "filtered by regular expression";
            wantsVerboseLogging = new RegExp(filter, "i"); // Case insensitive
        }
        else {
            RuntimePrivate_1.assertNever(filter, "Use true for verbose logging, false for default logging or a regular expression for filtering");
        }
        // eslint-disable-next-line no-console
        // tslint:disable-next-line:no-console
        console && console.log((ServiceDebugging.customLoggerPrefix || "") + `Service logging: ${mode}`);
        return wantsVerboseLogging;
    }
    ServiceDebugging.enableLogging = enableLogging;
    /** Customize logged service messages with an additional prefix string. */
    ServiceDebugging.customLoggerPrefix = "";
})(ServiceDebugging = exports.ServiceDebugging || (exports.ServiceDebugging = {}));
(function (ServiceDebugging) {
    /** Can be applied as a live debugging utility using Object.assign(window, ServiceDebugging.windowOverrides) . */
    ServiceDebugging.windowOverrides = {
        _enableServiceLogging: ServiceDebugging.enableLogging,
    };
})(ServiceDebugging = exports.ServiceDebugging || (exports.ServiceDebugging = {}));
let wantsVerboseLogging = false;
/**
 * @private Infrastructure that should never be used outside the Framer Services package implementation.
 */
(function (ServiceDebugging) {
    /** @private Create a new console-like logger object. */
    function createLogger(prefix = "", onWarn) {
        return {
            log: (message, ...optionalParams) => {
                if (wantsVerboseLogging === false)
                    return;
                if (wantsVerboseLogging !== true) {
                    if (typeof message !== "string")
                        return;
                    if (message.search(wantsVerboseLogging) < 0)
                        return;
                }
                if (typeof message === "object") {
                    optionalParams.unshift(message);
                    message = "";
                }
                const activeLogger = ServiceDebugging.customLogger || console;
                activeLogger && activeLogger.log((ServiceDebugging.customLoggerPrefix || "") + prefix + message, ...optionalParams);
            },
            warn: (message, ...optionalParams) => {
                if (onWarn) {
                    return onWarn(message);
                }
                if (typeof message === "object") {
                    optionalParams.unshift(message);
                    message = "";
                }
                const activeLogger = ServiceDebugging.customLogger || console;
                activeLogger &&
                    activeLogger.warn((ServiceDebugging.customLoggerPrefix || "") + prefix + " " + message, ...optionalParams);
            },
        };
    }
    ServiceDebugging.createLogger = createLogger;
    /** @private Only for tests. Enables or disables testing mode. */
    ServiceDebugging._isTesting = false;
    let _testingServiceManager;
    /** @private Only for tests. Temporarily changes the value of ServiceManager.shared(). */
    function _testWithShared(manager, fn) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!ServiceDebugging._isTesting)
                throw new Error("ServiceManager.isTesting must be true to use testWithShared()");
            if (_testingServiceManager)
                throw new Error("ServiceManager.testWithShared() may not be nested");
            try {
                _testingServiceManager = manager;
                return yield fn();
            }
            catch (error) {
                throw error;
            }
            finally {
                _testingServiceManager = undefined;
            }
        });
    }
    ServiceDebugging._testWithShared = _testWithShared;
    /** @private For ServiceManager */
    function _sharedServiceManagerIfTesting() {
        if (ServiceDebugging._isTesting) {
            if (!_testingServiceManager) {
                throw new Error("ServiceManager.shared() may not be used while testing. Use testWithShared() for explicitness.");
            }
            else {
                return _testingServiceManager;
            }
        }
        // Just use the standard shared logic if not testing
        return undefined;
    }
    ServiceDebugging._sharedServiceManagerIfTesting = _sharedServiceManagerIfTesting;
})(ServiceDebugging = exports.ServiceDebugging || (exports.ServiceDebugging = {}));
