"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const build_library_1 = require("@framerjs/build-library");
const interface_1 = require("../interface");
const validators = __importStar(require("../validation"));
const version_1 = require("../config/version");
const Api_1 = require("../Api");
exports.version = (argv) => __awaiter(void 0, void 0, void 0, function* () {
    interface_1.spinner.start("Validating");
    // Need FRAMER_TOKEN to setup .npmrc file correctly
    // which is needed to access information from the registry
    validators.framerTokenIsSet();
    // Build the configuration needed by the project
    const config = version_1.buildVersionConfig(argv);
    // Get the .npmrc template
    config.project.npmrcTemplate = yield new Api_1.API(config.api).getNpmrc();
    // Loading a new project will automatically install dependencies
    interface_1.spinner.text = "Setting up project";
    const project = yield build_library_1.Project.load(config.project);
    // Ensure there is sufficient data for checking the registry
    validators.canFetchPackageVersion(project);
    const packageName = yield project.getPackageName();
    interface_1.spinner.text = "Fetching package data";
    // Get package data from registry about the package
    let version;
    try {
        version = yield project.getLatestPublishedVersion();
    }
    catch (_a) {
        throw new Error(chalk_1.default `Cannot retrieve the latest version.\nPlease ensure the package was previously published.`);
    }
    interface_1.spinner.stopAndPersist({
        text: chalk_1.default `Latest version of {bold ${packageName}} is v{bold ${version}}.`,
        symbol: "🗂 ",
    });
});
