"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const fs_1 = require("../../fs");
const designDependenciesLoader = function designDependenciesLoader(source) {
    const callback = this.async();
    const sourceString = Buffer.isBuffer(source) ? source.toString("utf8") : source;
    const packageJsonPath = path_1.default.join(this.rootContext, "package.json");
    const yarnLockPath = path_1.default.join(this.rootContext, "yarn.lock");
    const nodeModulesPath = path_1.default.join(this.rootContext, "node_modules");
    // for the Build Service case we only care about updates to the dependencies installed with yarn
    // this might not be enough if we use "local" dependencies,
    // i.e. the ones located on the user file system somewhere next to the project
    // should that be a problem we might need to introduce a config option for this loader
    // that will add those "local" folders as loader dependencies
    this.addDependency(yarnLockPath);
    fs_1.readJSON(packageJsonPath).then((packageJson) => {
        // TODO: filter only Framer packages
        const dependencies = new Set([
            ...Object.keys(packageJson.dependencies || {}),
            ...Object.keys(packageJson.devDependencies || {}),
        ]);
        // TODO: make reads parallel
        //       there is an issue with async reads of files
        //       that is preventing this
        const designDependenciesTemplate = [];
        for (const dependency of dependencies) {
            let dependencyPath;
            if (packageJson.dependencies && packageJson.dependencies[dependency]) {
                dependencyPath = packageJson.dependencies[dependency];
            }
            else if (packageJson.devDependencies && packageJson.devDependencies[dependency]) {
                dependencyPath = packageJson.devDependencies[dependency];
            }
            let design = undefined;
            if (!design) {
                const moduleDesignPath = path_1.default.join(nodeModulesPath, dependency, "design", "document.json");
                design = fs_1.readFileSync(moduleDesignPath);
            }
            if (!design && dependencyPath) {
                const localDesignPath = path_1.default.join(this.rootContext, dependencyPath, "design", "document.json");
                design = fs_1.readFileSync(localDesignPath);
            }
            if (design) {
                designDependenciesTemplate.push(`designs["${dependency}"] = ${design}`);
            }
        }
        const content = sourceString.replace(/__{DESIGNS}__/g, designDependenciesTemplate.join("\n"));
        callback(null, content);
    }, error => {
        callback(error);
    });
};
// eslint-disable-next-line import/no-default-export
exports.default = designDependenciesLoader;
