import { ProjectPaths } from "../Project";
import { Package } from "../Package";
import { YarnCommandResult } from "../types";
import { updateInstallStrategy, installStrategy, generateActions, latestOfMajorVersions, newPeerDependency, isCompatible, installStrategyForVersion } from "./utils";
export { InstallStrategy, InstallType, Action } from "./utils";
declare type Paths = Pick<ProjectPaths, "project" | "nodeModules" | "yarnLinkDir" | "yarnLock">;
export interface FramerLibraryManagerConfig {
    paths: Paths;
    pkg: Package;
    builtInFramerVersion?: string | void;
}
export declare class FramerLibraryManager {
    private readonly paths;
    private readonly builtInFramerVersion?;
    private readonly pkg;
    static updateInstallStrategy: typeof updateInstallStrategy;
    static installStrategy: typeof installStrategy;
    static generateActions: typeof generateActions;
    static latestOfMajorVersions: typeof latestOfMajorVersions;
    static newPeerDependency: typeof newPeerDependency;
    static isCompatible: typeof isCompatible;
    static installStrategyForVersion: typeof installStrategyForVersion;
    constructor({ paths, pkg, builtInFramerVersion }: FramerLibraryManagerConfig);
    /**
     * @returns true if actions were performed to change the installed library
     */
    install(version?: string): Promise<boolean>;
    getPublishedFramerVersions(): Promise<YarnCommandResult>;
    private getPackageJson;
    private getPackageVersion;
    private installedFramerVersion;
    libraryCompatibility(): Promise<{
        compatible: boolean;
        isError: boolean;
        message: string;
        recoverySuggestion: string;
    }>;
    private resolvePackageFramerVersion;
}
