/// <reference types="node" />
import { CompilerConfig, Build } from "../Compiler";
import { YarnCommandResult, FramerPackageJson, InstalledPackage } from "../types";
import { PackageConfig } from "../Package/Package";
import { ProjectPathsConfig, ProjectPaths } from "./Paths";
export { ProjectPaths };
export interface ProjectSetupConfig extends ProjectPathsConfig {
    npmrcTemplate?: string;
    env?: NodeJS.ProcessEnv;
}
export interface ProjectConfig extends ProjectSetupConfig, Omit<PackageConfig, "paths"> {
    compiler?: Omit<CompilerConfig, "root">;
    builtInFramerVersion?: string;
}
export interface ProjectCreateConfig extends ProjectPathsConfig {
    templatePath: string;
}
export declare class Project {
    private readonly compiler;
    private readonly pkg;
    readonly path: string;
    readonly paths: ProjectPaths;
    static load(config: ProjectConfig): Promise<Project>;
    static create(config: ProjectCreateConfig): Promise<string>;
    private static prepareDir;
    private static extract;
    private constructor();
    install(): Promise<YarnCommandResult>;
    installFramer(version: string): Promise<void>;
    addDependency(dependency: string, version?: string): Promise<YarnCommandResult>;
    addPeerDependency(dependency: string, version?: string): Promise<YarnCommandResult>;
    addDevDependency(dependency: string, version?: string): Promise<YarnCommandResult>;
    removeDependency(dependency: string): Promise<YarnCommandResult>;
    getLatestPublishedVersion(): Promise<string>;
    build(): Promise<Build>;
    buildDist(): Promise<Build>;
    publish({ version, packageName, displayName, access, }: {
        version?: string | null;
        packageName?: string;
        displayName?: string;
        access?: "restricted" | "public";
    }): Promise<YarnCommandResult>;
    getDisplayName(): Promise<string>;
    getAuthor(): Promise<string>;
    getPackageName(): Promise<string>;
    getPackageVersion(): Promise<string>;
    getDependencies(): Promise<Pick<FramerPackageJson, Required<"dependencies"> | Required<"peerDependencies"> | Required<"devDependencies">>>;
    getPackages(...args: string[]): Promise<InstalledPackage[]>;
    setDisplayName(name: string): Promise<FramerPackageJson>;
    setAuthor(author: string): Promise<FramerPackageJson>;
    setPackageName(name: string): Promise<FramerPackageJson>;
    setPackageVersion(version: string): Promise<FramerPackageJson>;
    private writeBuildOutputToDisk;
    getTypingsForDependencies(): Promise<{
        module: string;
        content: string;
    }[]>;
    getTypingsForPackage(name: string): {
        module: string;
        content: string;
    }[];
    getTypingsAroundMain(module: string, modulepath: string): {
        module: string;
        content: string;
    }[];
    setYarnEnv(env: NodeJS.ProcessEnv): void;
}
