"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
function readJSONSync(filePath, shouldThrow = false) {
    try {
        const content = readFileSync(filePath, shouldThrow);
        if (!content)
            return {};
        return JSON.parse(content);
    }
    catch (error) {
        if (shouldThrow) {
            throw error;
        }
        return {};
    }
}
exports.readJSONSync = readJSONSync;
async function readJSON(filePath, shouldThrow = false) {
    try {
        const content = await readFile(filePath, shouldThrow);
        if (!content)
            return {};
        return JSON.parse(content);
    }
    catch (error) {
        if (shouldThrow) {
            throw error;
        }
        return {};
    }
}
exports.readJSON = readJSON;
async function readFile(filePath, shouldThrow = false) {
    try {
        return fs_extra_1.default.readFile(filePath, "utf8");
    }
    catch (error) {
        if (shouldThrow) {
            throw error;
        }
        return undefined;
    }
}
exports.readFile = readFile;
function readFileSync(filePath, shouldThrow = false) {
    try {
        return fs_extra_1.default.readFileSync(filePath, "utf8");
    }
    catch (error) {
        if (shouldThrow) {
            throw error;
        }
        return undefined;
    }
}
exports.readFileSync = readFileSync;
function removeSymbolicLink(pathToCheck, shouldThrow = false) {
    return fs_extra_1.default
        .stat(pathToCheck)
        .then(stats => (stats.isSymbolicLink ? fs_extra_1.default.remove(pathToCheck) : undefined))
        .catch(error => {
        if (shouldThrow) {
            throw error;
        }
    });
}
exports.removeSymbolicLink = removeSymbolicLink;
async function writeJSON(filePath, contents, shouldThrow = false) {
    try {
        await writeFile(filePath, JSON.stringify(contents, null, "\t"), shouldThrow);
    }
    catch (error) {
        if (shouldThrow) {
            throw error;
        }
    }
    return contents;
}
exports.writeJSON = writeJSON;
function writeJSONSync(filePath, contents, shouldThrow = false) {
    try {
        writeFileSync(filePath, JSON.stringify(contents, null, "\t"), shouldThrow);
    }
    catch (error) {
        if (shouldThrow) {
            throw error;
        }
    }
    return contents;
}
exports.writeJSONSync = writeJSONSync;
async function writeFile(filePath, contents, shouldThrow = false) {
    try {
        await fs_extra_1.default.writeFile(filePath, contents);
    }
    catch (error) {
        if (shouldThrow) {
            throw error;
        }
    }
    return contents;
}
exports.writeFile = writeFile;
function writeFileSync(filePath, contents, shouldThrow = false) {
    try {
        fs_extra_1.default.writeFileSync(filePath, contents);
    }
    catch (error) {
        if (shouldThrow) {
            throw error;
        }
    }
    return contents;
}
exports.writeFileSync = writeFileSync;
